/*-----copyright-----
 *     amy-studio - make development easier 
 *     web        : http://www.amy-studio.com
 *     taobao     : http://amy-studio.taobao.com
 *     qq(e_mail) : amystudio@qq.com
 *-----file info-----
 *     name       : disp_color_slip.v
 *     author     : Yuphone Chang
 *     description: generating the color slip
 *----- version -----
 *     created    : 7.20, 2010
 *     version    : 1.0
 *     description: original version
 *
 *     modified   : 
 *     version    : 
 *     description: 
 *-------------------
 */

module  disp_color_slip(
  input            i_clk,
  input            i_rst_n,
  input            i_switch,
  //
  input      [9:0] i_x_pos,
  input      [9:0] i_y_pos,
  output reg       o_red,
  output reg       o_green,
  output reg       o_blue  
);

// 800x600@72Hz
parameter X = 800;
parameter Y = 600;

//++++++++++++++++++++++++++++++++++++++
// Divide the X-axis into 16 level by the color density 
wire [3:0] pixel_x_buf =  (i_x_pos>=(X>>4)* 0 && i_x_pos<(X>>4)* 1) ? 4'h0 :
                          (i_x_pos>=(X>>4)* 1 && i_x_pos<(X>>4)* 2) ? 4'h1 :
                          (i_x_pos>=(X>>4)* 2 && i_x_pos<(X>>4)* 3) ? 4'h2 :
                          (i_x_pos>=(X>>4)* 3 && i_x_pos<(X>>4)* 4) ? 4'h3 :
                          (i_x_pos>=(X>>4)* 4 && i_x_pos<(X>>4)* 5) ? 4'h4 :
                          (i_x_pos>=(X>>4)* 5 && i_x_pos<(X>>4)* 6) ? 4'h5 :
                          (i_x_pos>=(X>>4)* 6 && i_x_pos<(X>>4)* 7) ? 4'h6 :
                          (i_x_pos>=(X>>4)* 7 && i_x_pos<(X>>4)* 8) ? 4'h7 :
                          (i_x_pos>=(X>>4)* 8 && i_x_pos<(X>>4)* 9) ? 4'h8 :
                          (i_x_pos>=(X>>4)* 9 && i_x_pos<(X>>4)*10) ? 4'h9 :
                          (i_x_pos>=(X>>4)*10 && i_x_pos<(X>>4)*11) ? 4'hA :
                          (i_x_pos>=(X>>4)*11 && i_x_pos<(X>>4)*12) ? 4'hB :
                          (i_x_pos>=(X>>4)*12 && i_x_pos<(X>>4)*13) ? 4'hC :
                          (i_x_pos>=(X>>4)*13 && i_x_pos<(X>>4)*14) ? 4'hD :
                          (i_x_pos>=(X>>4)*14 && i_x_pos<(X>>4)*15) ? 4'hE : 4'hF;

// Divide the Y-axis into 16 level by the color density                           
wire [3:0] pixel_y_buf =  (i_y_pos>=(Y>>4)* 0 && i_y_pos<(Y>>4)* 1) ? 4'h0 :
                          (i_y_pos>=(Y>>4)* 1 && i_y_pos<(Y>>4)* 2) ? 4'h1 :
                          (i_y_pos>=(Y>>4)* 2 && i_y_pos<(Y>>4)* 3) ? 4'h2 :
                          (i_y_pos>=(Y>>4)* 3 && i_y_pos<(Y>>4)* 4) ? 4'h3 :
                          (i_y_pos>=(Y>>4)* 4 && i_y_pos<(Y>>4)* 5) ? 4'h4 :
                          (i_y_pos>=(Y>>4)* 5 && i_y_pos<(Y>>4)* 6) ? 4'h5 :
                          (i_y_pos>=(Y>>4)* 6 && i_y_pos<(Y>>4)* 7) ? 4'h6 :
                          (i_y_pos>=(Y>>4)* 7 && i_y_pos<(Y>>4)* 8) ? 4'h7 :
                          (i_y_pos>=(Y>>4)* 8 && i_y_pos<(Y>>4)* 9) ? 4'h8 :
                          (i_y_pos>=(Y>>4)* 9 && i_y_pos<(Y>>4)*10) ? 4'h9 :
                          (i_y_pos>=(Y>>4)*10 && i_y_pos<(Y>>4)*11) ? 4'hA :
                          (i_y_pos>=(Y>>4)*11 && i_y_pos<(Y>>4)*12) ? 4'hB :
                          (i_y_pos>=(Y>>4)*12 && i_y_pos<(Y>>4)*13) ? 4'hC :
                          (i_y_pos>=(Y>>4)*13 && i_y_pos<(Y>>4)*14) ? 4'hD :
                          (i_y_pos>=(Y>>4)*14 && i_y_pos<(Y>>4)*15) ? 4'hE : 4'hF;
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
always @ (posedge i_clk, negedge i_rst_n)
begin
  if (!i_rst_n)                         // reset to send black color 
    {o_red, o_green, o_blue} <= 3'b000;
  else
  begin
  
    
    //++++++++++++++++++++++++++++++++++
    // generating 16-vertical and 4-horizontal blocks begins  
    //++++++++++++++++++++++++++++++++++
    if (i_switch)                       // SW[1] is close?
    begin                               // Divide the Y-axis into 4 level 
      if (i_y_pos>=(Y>>2)*0 && i_y_pos<(Y>>2)*1)
      begin
        o_red   <= pixel_x_buf;
        o_green <= 1'b0; 
        o_blue  <= 1'b0;
      end
      else if (i_y_pos>=(Y>>2)*1 && i_y_pos<(Y>>2)*2)
      begin
        o_red   <= 1'b0;
        o_green <= pixel_x_buf;
        o_blue  <= 1'b0;
      end
      else if (i_y_pos>=(Y>>2)*2 && i_y_pos<(Y>>2)*3)
      begin
        o_red   <= 1'b0;
        o_green <= 1'b0;
        o_blue  <= pixel_x_buf;
      end
      else
      begin
        o_red   <= pixel_x_buf;                  
        o_green <= pixel_x_buf;         
        o_blue  <= pixel_x_buf;
      end
    end      
    //----------------------------------
    // generating 16-vertical and 4-horizontal blocks ends 
    //----------------------------------
    
    
    //++++++++++++++++++++++++++++++++++
    // generating 4-vertical and 16-horizontal blocks begins 
    //++++++++++++++++++++++++++++++++++
    else                                // SW[1] is open?
    begin                               // Divide the X-axis into 4 level  
      if (i_x_pos>=(X>>2)*0 && i_x_pos<(X>>2)*1)
      begin
        o_red   <= pixel_y_buf;
        o_green <= 1'b0; 
        o_blue  <= 1'b0;
      end
      else if (i_x_pos>=(X>>2)*1 && i_x_pos<(X>>2)*2)
      begin
        o_red   <= 1'b0;
        o_green <= pixel_y_buf;
        o_blue  <= 1'b0;
      end
      else if (i_x_pos>=(X>>2)*2 && i_x_pos<(X>>2)*3)
      begin
        o_red   <= 1'b0;
        o_green <= 1'b0;   
        o_blue  <= pixel_y_buf;
      end
      else
      begin
        o_red   <= pixel_y_buf;
        o_green <= pixel_y_buf; 
        o_blue  <= pixel_y_buf;
      end
    end
    //----------------------------------
    // generating 4-vertical and 16-horizontal blocks ends 
    //----------------------------------
    
  end
end
//--------------------------------------

endmodule
